@@ -46,9 +46,8 @@ class FailedToLoadTask(exceptions.RallyException):
 
 
 class TaskCommands(object):
-    """Task management.
+    """Set of commands that allow you to manage benchmarking tasks and results.
 
-    Set of commands that allow you to manage benchmarking tasks and results.
     """
 
     def _load_task(self, task_file, task_args=None, task_args_file=None):
@@ -138,17 +137,18 @@ class TaskCommands(object):
         print(_("Task config is valid :)"))
         return input_task
 
-    @cliutils.args("--deployment", type=str, dest="deployment",
-                   required=False, help="UUID or name of the deployment")
-    @cliutils.args("--task", "--filename",
-                   help="Path to the file with full configuration of task")
-    @cliutils.args("--task-args", dest="task_args",
-                   help="Input task args (dict in json). These args are used "
-                        "to render input task that is jinja2 template.")
-    @cliutils.args("--task-args-file", dest="task_args_file",
+    @cliutils.args("--deployment", dest="deployment", type=str,
+                   metavar="<uuid>", required=False,
+                   help="UUID or name of a deployment.")
+    @cliutils.args("--task", "--filename", metavar="<path>",
+                   help="Path to the input task file.")
+    @cliutils.args("--task-args", metavar="<json>", dest="task_args",
+                   help="Input task args (JSON dict). These args are used "
+                        "to render the Jinja2 template in the input task.")
+    @cliutils.args("--task-args-file", metavar="<path>", dest="task_args_file",
                    help="Path to the file with input task args (dict in "
-                        "json/yaml). These args are used to render input "
-                        "task that is jinja2 template.")
+                        "JSON/YAML). These args are used "
+                        "to render the Jinja2 template in the input task.")
     @envutils.with_default_deployment(cli_arg_name="deployment")
     @plugins.ensure_plugins_are_loaded
     def validate(self, task, deployment=None, task_args=None,
@@ -158,13 +158,19 @@ class TaskCommands(object):
         This will check that task configuration file has valid syntax and
         all required options of scenarios, contexts, SLA and runners are set.
 
-        :param task: a file with yaml/json task
-        :param task_args: Input task args (dict in json/yaml). These args are
-                          used to render input task that is jinja2 template.
-        :param task_args_file: File with input task args (dict in json/yaml).
-                               These args are used to render input task that
-                               is jinja2 template.
-        :param deployment: UUID or name of a deployment
+        If both task_args and task_args_file are specified, they will
+        be merged. task_args has a higher priority so it will override
+        values from task_args_file.
+
+        :param task: Path to the input task file.
+        :param task_args: Input task args (JSON dict). These args are
+                          used to render the Jinja2 template in the
+                          input task.
+        :param task_args_file: Path to the file with input task args
+                               (dict in JSON/YAML). These args are
+                               used to render the Jinja2 template in
+                               the input task.
+        :param deployment: UUID or name of the deployment
         """
         try:
             self._load_and_validate_task(task, task_args, task_args_file,
@@ -174,36 +180,44 @@ class TaskCommands(object):
             print(e, file=sys.stderr)
             return(1)
 
-    @cliutils.args("--deployment", type=str, dest="deployment",
-                   required=False, help="UUID or name of the deployment")
-    @cliutils.args("--task", "--filename", help="Path to the input task file")
-    @cliutils.args("--task-args", dest="task_args",
-                   help="Input task args (dict in json). These args are used "
-                        "to render input task that is jinja2 template.")
-    @cliutils.args("--task-args-file", dest="task_args_file",
+    @cliutils.args("--deployment", dest="deployment", type=str,
+                   metavar="<uuid>", required=False,
+                   help="UUID or name of a deployment.")
+    @cliutils.args("--task", "--filename", metavar="<path>",
+                   help="Path to the input task file")
+    @cliutils.args("--task-args", dest="task_args", metavar="<json>",
+                   help="Input task args (JSON dict). These args are used "
+                        "to render the Jinja2 template in the input task.")
+    @cliutils.args("--task-args-file", dest="task_args_file", metavar="<path>",
                    help="Path to the file with input task args (dict in "
-                        "json/yaml). These args are used to render input "
-                        "task that is jinja2 template.")
+                        "JSON/YAML). These args are used "
+                        "to render the Jinja2 template in the input task.")
     @cliutils.args("--tag", help="Tag for this task")
     @cliutils.args("--no-use", action="store_false", dest="do_use",
-                   help="Don't set new task as default for future operations")
+                   help="Don't set new task as default for future operations.")
     @cliutils.args("--abort-on-sla-failure", action="store_true",
                    dest="abort_on_sla_failure",
                    help="Abort the execution of a benchmark scenario when"
-                        "any SLA check for it fails")
+                        "any SLA check for it fails.")
     @envutils.with_default_deployment(cli_arg_name="deployment")
     @plugins.ensure_plugins_are_loaded
     def start(self, task, deployment=None, task_args=None, task_args_file=None,
               tag=None, do_use=False, abort_on_sla_failure=False):
         """Start benchmark task.
 
-        :param task: a file with yaml/json task
-        :param task_args: Input task args (dict in json/yaml). These args are
-                          used to render input task that is jinja2 template.
-        :param task_args_file: File with input task args (dict in json/yaml).
-                               These args are used to render input task that
-                               is jinja2 template.
-        :param deployment: UUID or name of a deployment
+        If both task_args and task_args_file are specified, they will
+        be merged. task_args has a higher priority so it will override
+        values from task_args_file.
+
+        :param task: Path to the input task file.
+        :param task_args: Input task args (JSON dict). These args are
+                          used to render the Jinja2 template in the
+                          input task.
+        :param task_args_file: Path to the file with input task args
+                               (dict in JSON/YAML). These args are
+                               used to render the Jinja2 template in
+                               the input task.
+        :param deployment: UUID or name of the deployment
         :param tag: optional tag for this task
         :param do_use: if True, the new task will be stored as the default one
                        for future operations
@@ -239,22 +253,23 @@ class TaskCommands(object):
             print(e, file=sys.stderr)
             return(1)
 
-    @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task")
+    @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task.")
     @envutils.with_default_task_id
-    @cliutils.args("--soft", action="store_true",
-                   help="Abort task after current scenario full execution")
+    @cliutils.args(
+        "--soft", action="store_true",
+        help="Abort task after current scenario finishes execution.")
     def abort(self, task_id=None, soft=False):
-        """Abort started benchmarking task.
+        """Abort a running benchmarking task.
 
         :param task_id: Task uuid
         :param soft: if set to True, task should be aborted after execution of
                      current scenario
         """
         if soft:
-            print("INFO: please be informed that soft abort wont stop "
-                  "current running scenario, it will prevent to start "
-                  "new ones, so if you are running task with only one "
-                  "scenario - soft abort will not help at all.")
+            print("INFO: please be informed that soft abort won't stop "
+                  "a running scenario, but will prevent new ones from "
+                  "starting. If you are running task with only one "
+                  "scenario, soft abort will not help at all.")
 
         api.Task.abort(task_id, soft, async=False)
 
@@ -263,7 +278,7 @@ class TaskCommands(object):
     @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task")
     @envutils.with_default_task_id
     def status(self, task_id=None):
-        """Display current status of task.
+        """Display the current status of a task.
 
         :param task_id: Task uuid
         Returns current status of task
@@ -274,11 +289,11 @@ class TaskCommands(object):
               % {"task_id": task_id, "status": task["status"]})
 
     @cliutils.args("--uuid", type=str, dest="task_id",
-                   help=("uuid of task, if --uuid is \"last\" results of most "
-                         "recently created task will be displayed."))
+                   help=("UUID of task. If --uuid is \"last\" the results of "
+                         " the most recently created task will be displayed."))
     @cliutils.args("--iterations-data", dest="iterations_data",
                    action="store_true",
-                   help="print detailed results for each iteration")
+                   help="Print detailed results for each iteration.")
     @envutils.with_default_task_id
     def detailed(self, task_id=None, iterations_data=False):
         """Display results table.
@@ -393,25 +408,52 @@ class TaskCommands(object):
             raw = result["data"]["raw"]
             # NOTE(hughsaunders): ssrs=scenario specific results
             ssrs = []
-            for result in raw:
-                data = result["scenario_output"].get("data")
-                if data:
-                    ssrs.append(data)
-            if ssrs:
-                keys = set()
-                for ssr in ssrs:
-                    keys.update(ssr.keys())
-                headers = ["key", "min", "median",
-                           "90%ile", "95%ile", "max",
-                           "avg"]
-                float_cols = ["min", "median", "90%ile",
-                              "95%ile", "max", "avg"]
-                formatters = dict(zip(float_cols,
-                                  [cliutils.pretty_float_formatter(col, 3)
-                                   for col in float_cols]))
-                table_rows = []
-                for key in keys:
-                    values = [float(ssr[key]) for ssr in ssrs if key in ssr]
+            for itr in raw:
+                if "output" not in itr:
+                    itr["output"] = {"additive": [], "complete": []}
+
+                    # NOTE(amaretskiy): "scenario_output" is supported
+                    #   for backward compatibility
+                    if ("scenario_output" in itr
+                            and itr["scenario_output"]["data"]):
+                        itr["output"]["additive"].append(
+                            {"data": itr["scenario_output"]["data"].items(),
+                             "title": "Scenario output",
+                             "description": "",
+                             "chart_plugin": "StackedArea"})
+                        del itr["scenario_output"]
+
+                for idx, additive in enumerate(itr["output"]["additive"]):
+                    try:
+                        for key, value in additive["data"]:
+                            ssrs[idx]["data"][key].append(value)
+                    except IndexError:
+                        data = {}
+                        keys = []
+                        for key, value in additive["data"]:
+                            if key not in data:
+                                data[key] = []
+                                keys.append(key)
+                            data[key].append(value)
+                        ssrs.append({"title": additive["title"],
+                                     "keys": keys,
+                                     "data": data})
+            if not ssrs:
+                return
+
+            print("\nScenario Specific Results\n")
+
+            headers = ["key", "min", "median", "90%ile", "95%ile",
+                       "max", "avg"]
+            float_cols = ["min", "median", "90%ile", "95%ile", "max", "avg"]
+            formatters = dict(zip(float_cols,
+                              [cliutils.pretty_float_formatter(col, 3)
+                               for col in float_cols]))
+
+            for ssr in ssrs:
+                rows = []
+                for key in ssr["keys"]:
+                    values = ssr["data"][key]
 
                     if values:
                         row = [str(key),
@@ -423,18 +465,13 @@ class TaskCommands(object):
                                round(utils.mean(values), 3)]
                     else:
                         row = [str(key)] + ["n/a"] * 6
-                    table_rows.append(rutils.Struct(**dict(zip(headers,
-                                                               row))))
-                print("\nScenario Specific Results\n")
-                cliutils.print_list(table_rows,
+                    rows.append(rutils.Struct(**dict(zip(headers, row))))
+
+                cliutils.print_list(rows,
                                     fields=headers,
                                     formatters=formatters,
-                                    table_label="Response Times (sec)")
-
-                for result in raw:
-                    errors = result["scenario_output"].get("errors")
-                    if errors:
-                        print(errors)
+                                    table_label=ssr["title"])
+                print()
 
         def _print_hints(task):
             print()
@@ -468,7 +505,7 @@ class TaskCommands(object):
                       result["data"]["full_duration"])
             _print_hints(task)
 
-    @cliutils.args("--uuid", type=str, dest="task_id", help="uuid of task")
+    @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task.")
     @envutils.with_default_task_id
     @cliutils.suppress_warnings
     def results(self, task_id=None):
@@ -492,9 +529,9 @@ class TaskCommands(object):
                 task_id, consts.TaskStatus.FAILED, consts.TaskStatus.FINISHED))
             return(1)
 
-    @cliutils.args("--deployment", type=str, dest="deployment",
-                   help="List tasks from specified deployment."
-                   "By default tasks listed from active deployment.")
+    @cliutils.args("--deployment", dest="deployment", type=str,
+                   metavar="<uuid>", required=False,
+                   help="UUID or name of a deployment.")
     @cliutils.args("--all-deployments", action="store_true",
                    dest="all_deployments",
                    help="List tasks from all deployments.")
@@ -502,15 +539,16 @@ class TaskCommands(object):
                    help="List tasks with specified status."
                    " Available statuses: %s" % ", ".join(consts.TaskStatus))
     @cliutils.args("--uuids-only", action="store_true",
-                   dest="uuids_only", help="List task UUIDs only")
+                   dest="uuids_only", help="List task UUIDs only.")
     @envutils.with_default_deployment(cli_arg_name="deployment")
     def list(self, deployment=None, all_deployments=False, status=None,
              uuids_only=False):
         """List tasks, started and finished.
 
-        Displayed tasks could be filtered by status or deployment.
-        By default 'rally task list' will display tasks from active deployment
-        without filtering by status.
+        Displayed tasks can be filtered by status or deployment.  By
+        default 'rally task list' will display tasks from the active
+        deployment without filtering by status.
+
         :param deployment: UUID or name of deployment
         :param status: task status to filter by.
             Available task statuses are in rally.consts.TaskStatus
@@ -558,11 +596,12 @@ class TaskCommands(object):
                         "\trally task start"))
 
     @cliutils.args("--tasks", dest="tasks", nargs="+",
-                   help="uuids of tasks or json files with task results")
-    @cliutils.args("--out", type=str, dest="out", required=True,
+                   help="UUIDs of tasks, or JSON files with task results")
+    @cliutils.args("--out", metavar="<path>",
+                   type=str, dest="out", required=False,
                    help="Path to output file.")
     @cliutils.args("--open", dest="open_it", action="store_true",
-                   help="Open it in browser.")
+                   help="Open the output in a browser.")
     @cliutils.args("--html", dest="out_format",
                    action="store_const", const="html",
                    help="Generate the report in HTML.")
@@ -570,11 +609,11 @@ class TaskCommands(object):
                    action="store_const", const="html_static",
                    help=("Generate the report in HTML with embedded "
                          "JS and CSS, so it will not depend on "
-                         "the Internet availability."))
+                         "Internet availability."))
     @cliutils.args("--junit", dest="out_format",
                    action="store_const", const="junit",
                    help="Generate the report in the JUnit format.")
-    @envutils.default_from_global("tasks", envutils.ENV_TASK, "uuid")
+    @envutils.default_from_global("tasks", envutils.ENV_TASK, "tasks")
     @cliutils.suppress_warnings
     def report(self, tasks=None, out=None, open_it=False, out_format="html"):
         """Generate report file for specified task.
@@ -630,14 +669,9 @@ class TaskCommands(object):
                     processed_names[task_result["key"]["name"]] = 0
                 results.append(task_result)
 
-        output_file = os.path.expanduser(out)
-
         if out_format.startswith("html"):
-            with open(output_file, "w+") as f:
-                f.write(plot.plot(results,
-                                  include_libs=(out_format == "html_static")))
-            if open_it:
-                webbrowser.open_new_tab("file://" + os.path.realpath(out))
+            result = plot.plot(results,
+                               include_libs=(out_format == "html_static"))
         elif out_format == "junit":
             test_suite = junit.JUnit("Rally test suite")
             for result in results:
@@ -650,17 +684,25 @@ class TaskCommands(object):
                     outcome = junit.JUnit.SUCCESS
                 test_suite.add_test(result["key"]["name"],
                                     result["full_duration"], outcome, message)
-            with open(output_file, "w+") as f:
-                f.write(test_suite.to_xml())
+            result = test_suite.to_xml()
         else:
-            print(_("Invalid output format: %s") % out_format,
-                  file=sys.stderr)
+            print(_("Invalid output format: %s") % out_format, file=sys.stderr)
             return 1
 
+        if out:
+            output_file = os.path.expanduser(out)
+
+            with open(output_file, "w+") as f:
+                f.write(result)
+            if open_it:
+                webbrowser.open_new_tab("file://" + os.path.realpath(out))
+        else:
+            print(result)
+
     @cliutils.args("--force", action="store_true", help="force delete")
     @cliutils.args("--uuid", type=str, dest="task_id", nargs="*",
                    metavar="TASK_ID",
-                   help="uuid of task or a list of task uuids")
+                   help="UUID of task or a list of task UUIDs.")
     @envutils.with_default_task_id
     def delete(self, task_id=None, force=False):
         """Delete task and its results.
@@ -684,10 +726,10 @@ class TaskCommands(object):
         else:
             _delete_single_task(task_id, force)
 
-    @cliutils.args("--uuid", type=str, dest="task_id", help="uuid of task")
+    @cliutils.args("--uuid", type=str, dest="task_id", help="UUID of task.")
     @cliutils.args("--json", dest="tojson",
                    action="store_true",
-                   help="output in json format")
+                   help="Output in JSON format.")
     @envutils.with_default_task_id
     def sla_check(self, task_id=None, tojson=False):
         """Display SLA check results table.
@@ -717,13 +759,15 @@ class TaskCommands(object):
                                        "status", "detail"))
         return failed_criteria
 
-    @cliutils.args("--task", type=str, dest="task", required=False,
+    @cliutils.args("--uuid", type=str, dest="task_id",
                    help="UUID of the task")
-    def use(self, task):
+    @cliutils.deprecated_args("--task", dest="task_id", type=str,
+                              release="0.2.0", alternative="--uuid")
+    def use(self, task_id):
         """Set active task.
 
-        :param task: Task uuid.
+        :param task_id: Task uuid.
         """
-        print("Using task: %s" % task)
-        api.Task.get(task)
-        fileutils.update_globals_file("RALLY_TASK", task)
+        print("Using task: %s" % task_id)
+        api.Task.get(task_id)
+        fileutils.update_globals_file("RALLY_TASK", task_id)
