@@ -30,6 +30,7 @@ from rally.common import objects
 from rally.common import utils
 from rally import exceptions
 from rally import osclients
+from rally.plugins.openstack.wrappers import glance as glance_wrapper
 from rally.plugins.openstack.wrappers import network
 
 LOG = logging.getLogger(__name__)
@@ -337,10 +338,10 @@ class TempestResourcesContext(utils.RandomNameGeneratorMixin):
                                self._discover_or_create_image)
         self._configure_option("compute", "image_ref_alt",
                                self._discover_or_create_image)
-        self._configure_option("compute",
-                               "flavor_ref", self._create_flavor, 64)
-        self._configure_option("compute",
-                               "flavor_ref_alt", self._create_flavor, 128)
+        self._configure_option("compute", "flavor_ref",
+                               self._discover_or_create_flavor, 64)
+        self._configure_option("compute", "flavor_ref_alt",
+                               self._discover_or_create_flavor, 128)
         if "neutron" in self.available_services:
             neutronclient = self.clients.neutron()
             if neutronclient.list_networks(shared=True)["networks"]:
@@ -357,7 +358,7 @@ class TempestResourcesContext(utils.RandomNameGeneratorMixin):
                                        self._create_network_resources)
         if "heat" in self.available_services:
             self._configure_option("orchestration", "instance_type",
-                                   self._create_flavor, 64)
+                                   self._discover_or_create_flavor, 64)
 
         _write_config(self.conf_path, self.conf)
 
@@ -406,13 +407,13 @@ class TempestResourcesContext(utils.RandomNameGeneratorMixin):
                       .format(opt=option, opt_val=option_value))
 
     def _discover_or_create_image(self):
-        glanceclient = self.clients.glance()
+        glance_wrap = glance_wrapper.wrap(self.clients.glance, self)
 
         if CONF.image.name_regex:
             LOG.debug("Trying to discover an image with name matching "
                       "regular expression '%s'. Note that case insensitive "
                       "matching is performed" % CONF.image.name_regex)
-            img_list = [img for img in glanceclient.images.list()
+            img_list = [img for img in self.clients.glance().images.list()
                         if img.status.lower() == "active" and img.name]
             for img in img_list:
                 if re.match(CONF.image.name_regex, img.name, re.IGNORECASE):
@@ -427,25 +428,38 @@ class TempestResourcesContext(utils.RandomNameGeneratorMixin):
             "name": self.generate_random_name(),
             "disk_format": CONF.image.disk_format,
             "container_format": CONF.image.container_format,
+            "image_location": os.path.join(_create_or_get_data_dir(),
+                                           self.image_name),
             "is_public": True
         }
         LOG.debug("Creating image '%s'" % params["name"])
-        image = glanceclient.images.create(**params)
+        image = glance_wrap.create_image(**params)
         self._created_images.append(image)
-        image.update(data=open(
-            os.path.join(_create_or_get_data_dir(), self.image_name), "rb"))
 
         return image
 
-    def _create_flavor(self, flv_ram):
+    def _discover_or_create_flavor(self, flv_ram):
         novaclient = self.clients.nova()
+
+        LOG.debug("Trying to discover a flavor with the following "
+                  "properties: RAM = %dMB, VCPUs = 1, disk = 0GB" % flv_ram)
+        for flavor in novaclient.flavors.list():
+            if (flavor.ram == flv_ram
+                    and flavor.vcpus == 1 and flavor.disk == 0):
+                LOG.debug("The following flavor discovered: '{0}'. Using "
+                          "flavor '{0}' for the tests".format(flavor.name))
+                return flavor
+
+        LOG.debug("There is no flavor with the mentioned properties")
+
         params = {
             "name": self.generate_random_name(),
             "ram": flv_ram,
             "vcpus": 1,
             "disk": 0
         }
-        LOG.debug("Creating flavor '%s'" % params["name"])
+        LOG.debug("Creating flavor '%s' with the following properties: RAM "
+                  "= %dMB, VCPUs = 1, disk = 0GB" % (params["name"], flv_ram))
         flavor = novaclient.flavors.create(**params)
         self._created_flavors.append(flavor)
 
