@@ -409,7 +409,8 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
 
         self.tempest.is_installed.assert_called_once_with()
         self.tempest.verify.assert_called_once_with(
-            set_name="smoke", regex=None, tests_file=None, concur=0)
+            set_name="smoke", regex=None, tests_file=None,
+            expected_failures=None, concur=0)
 
     @mock.patch("rally.api.objects.Deployment.get")
     @mock.patch("rally.api.objects.Verification")
@@ -427,7 +428,8 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
         self.tempest.is_installed.assert_called_once_with()
         self.tempest.install.assert_called_once_with()
         self.tempest.verify.assert_called_once_with(
-            set_name="smoke", regex=None, tests_file=None, concur=0)
+            set_name="smoke", regex=None, tests_file=None,
+            expected_failures=None, concur=0)
 
     @mock.patch("os.path.exists", return_value=True)
     @mock.patch("rally.api.objects.Deployment.get")
@@ -444,7 +446,8 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
             tests_file=tests_file, tempest_config=None)
 
         self.tempest.verify.assert_called_once_with(
-            set_name="", regex=None, tests_file=tests_file, concur=0)
+            set_name="", regex=None, tests_file=tests_file,
+            expected_failures=None, concur=0)
 
     @mock.patch("rally.common.objects.Deployment.get")
     @mock.patch("rally.api.objects.Verification")
@@ -497,6 +500,13 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
         self.tempest.install.assert_called_once_with()
         mock_move.assert_called_once_with(tmp_file, fake_conf)
 
+    @mock.patch("rally.common.objects.Deployment.get")
+    @mock.patch("rally.verification.tempest.tempest.Tempest")
+    def test_discover_tests(self, mock_tempest, mock_deployment_get):
+        mock_tempest.return_value = self.tempest
+        api.Verification.discover_tests(self.deployment_uuid, "some_pattern")
+        self.tempest.discover_tests.assert_called_once_with("some_pattern")
+
     @mock.patch("os.path.exists", return_value=True)
     @mock.patch("rally.common.objects.Deployment.get")
     @mock.patch("rally.verification.tempest.tempest.Tempest")
@@ -555,7 +565,7 @@ class VerificationAPITestCase(BaseDeploymentTestCase):
         self.assertEqual(0, mock_open.call_count)
 
     @mock.patch("rally.common.objects.Verification.list")
-    def test_delete(self, mock_verification_list):
+    def test_list(self, mock_verification_list):
         retval = api.Verification.list()
         self.assertEqual(mock_verification_list.return_value, retval)
         mock_verification_list.assert_called_once_with(None)
